@@ -1,5 +1,7 @@
 from __future__ import annotations
 
+import pytest
+
 import narwhals as nw
 from tests.utils import ConstructorEager, assert_equal_data
 
@@ -12,7 +14,12 @@ def test_map_batches_expr(constructor_eager: ConstructorEager) -> None:
     assert_equal_data(expected, {"a": [2, 3, 4], "b": [5, 6, 7]})
 
 
-def test_map_batches_expr_numpy(constructor_eager: ConstructorEager) -> None:
+def test_map_batches_expr_numpy(
+    constructor_eager: ConstructorEager, request: pytest.FixtureRequest
+) -> None:
+    if "polars" in str(constructor_eager):
+        # https://github.com/narwhals-dev/narwhals/issues/2995
+        request.applymarker(pytest.mark.xfail(strict=False))
     df = nw.from_native(constructor_eager(data))
     expected = df.select(
         nw.col("a")
